@@ -36,6 +36,12 @@ import urllib
 
 from types import ModuleType
 from base64 import encodebytes
+try:
+    import resource
+except ImportError:
+    # Windows
+    resource = None
+
 from jinja2 import Environment, FileSystemLoader
 
 from jupyter_server.transutils import trans, _
@@ -758,6 +764,30 @@ class ServerApp(JupyterApp):
             self._token_generated = True
             return binascii.hexlify(os.urandom(24)).decode('ascii')
 
+    min_open_files_limit = Integer(config=True,
+        help="""
+        Gets or sets a lower bound on the open file handles process resource
+        limit. This may need to be increased if you run into an
+        OSError: [Errno 24] Too many open files.
+        This is not applicable when running on Windows.
+        """)
+
+    @default('min_open_files_limit')
+    def _default_min_open_files_limit(self):
+        if resource is None:
+            # Ignoring min_open_files_limit because the limit cannot be adjusted (for example, on Windows)
+            return None
+
+        soft, hard = resource.getrlimit(resource.RLIMIT_NOFILE)
+
+        DEFAULT_SOFT = 4096
+        if hard >= DEFAULT_SOFT:
+            return DEFAULT_SOFT
+
+        self.log.debug("Default value for min_open_files_limit is ignored (hard=%r, soft=%r)", hard, soft)
+
+        return soft
+
     max_body_size = Integer(512 * 1024 * 1024, config=True,
         help="""
         Sets the maximum allowed size of the client request body, specified in
@@ -879,6 +909,22 @@ class ServerApp(JupyterApp):
         else:
             return not addr.is_loopback
 
+    use_redirect_file = Bool(True, config=True,
+        help="""Disable launching browser by redirect file
+     For versions of notebook > 5.7.2, a security feature measure was added that
+     prevented the authentication token used to launch the browser from being visible.
+     This feature makes it difficult for other users on a multi-user system from
+     running code in your Jupyter session as you.
+     However, some environments (like Windows Subsystem for Linux (WSL) and Chromebooks),
+     launching a browser using a redirect file can lead the browser failing to load.
+     This is because of the difference in file structures/paths between the runtime and
+     the browser.
+
+     Disabling this setting to False will disable this behavior, allowing the browser
+     to launch by using a URL and visible token (as before).
+     """
+    )
+
     local_hostnames = List(Unicode(), ['localhost'], config=True,
        help="""Hostnames to allow as local when allow_remote_access is False.
 
@@ -887,6 +933,28 @@ class ServerApp(JupyterApp):
        """
     )
 
+    # The name of the app that started this server (if not started directly).
+    # It is sometimes important to know if + which another app (say a server extension)
+    # started the serverapp to properly configure some traits.
+    # This trait should not be configured by users. It will likely be set by ExtensionApp.
+    _starter_app_name = Unicode(None, allow_none=True)
+
+    @validate('_starter_app_name')
+    def _validate_starter_app(self, proposal):
+        # Check that a previous server extension isn't named yet
+        value = proposal["value"]
+        if self._starter_app_name != None:
+            raise TraitError("Another extension was already named as the starter_server_extension.")
+        return value
+
+    @property
+    def starter_app(self):
+        """Get the Extension that started this server."""
+        name = self._starter_app_name
+        if name is None:
+            return
+        return self.extension_manager.extension_points.get(name, None).app
+
     open_browser = Bool(False, config=True,
                         help="""Whether to open in a browser after starting.
                         The specific browser used is platform dependent and
@@ -895,6 +963,31 @@ class ServerApp(JupyterApp):
                         (ServerApp.browser) configuration option.
                         """)
 
+
+    def _handle_browser_opening(self):
+        """This method handles whether a browser should be opened.
+        By default, Jupyter Server doesn't try to open an browser. However,
+        it's many server extensions might want to open the browser by default.
+        This essentially toggles the default value for open_browser.
+
+        From a UX perspective, this needs to be surfaced to the user. The default
+        behavior of Jupyter Server switches, which can be confusing.
+        """
+        # If the server was started by another application, use that applications
+        # trait for the open_browser trait. If that trait is not given, ignore
+        if self.starter_app:
+            try:
+                if self.starter_app.open_browser:
+                    self.launch_browser()
+            # If the starter_app doesn't have an open_browser trait, ignore
+            # move on and don't start a browser.
+            except AttributeError:
+                pass
+        else:
+            if self.open_browser:
+                self.launch_browser()
+
+
     browser = Unicode(u'', config=True,
                       help="""Specify what command to use to invoke a web
                       browser when starting the server. If not specified, the
@@ -1332,6 +1425,23 @@ class ServerApp(JupyterApp):
 
         self.login_handler_class.validate_security(self, ssl_options=self.ssl_options)
 
+    def init_resources(self):
+        """initialize system resources"""
+        if resource is None:
+            self.log.debug('Ignoring min_open_files_limit because the limit cannot be adjusted (for example, on Windows)')
+            return
+
+        old_soft, old_hard = resource.getrlimit(resource.RLIMIT_NOFILE)
+        soft = self.min_open_files_limit
+        hard = old_hard
+        if old_soft < soft:
+            if hard < soft:
+                hard = soft
+            self.log.debug(
+                'Raising open file limit: soft {}->{}; hard {}->{}'.format(old_soft, soft, old_hard, hard)
+            )
+            resource.setrlimit(resource.RLIMIT_NOFILE, (soft, hard))
+
     @property
     def display_url(self):
         if self.custom_display_url:
@@ -1649,6 +1759,7 @@ class ServerApp(JupyterApp):
             self.find_server_extensions()
         self.init_logging()
         self.init_server_extensions()
+        self.init_resources()
         self.init_configurables()
         self.init_components()
         self.init_webapp()
@@ -1761,6 +1872,12 @@ class ServerApp(JupyterApp):
         if not browser:
             return
 
+        if not self.use_redirect_file:
+            uri = self.default_url[len(self.base_url):]
+
+            if self.token:
+                uri = url_concat(uri, {'token': self.token})
+
         if self.file_to_run:
             if not os.path.exists(self.file_to_run):
                 self.log.critical(_("%s does not exist") % self.file_to_run)
@@ -1776,9 +1893,12 @@ class ServerApp(JupyterApp):
         else:
             open_file = self.browser_open_file
 
-        b = lambda: browser.open(
-            urljoin('file:', pathname2url(open_file)),
-            new=self.webbrowser_open_new)
+        if self.use_redirect_file:
+            assembled_url = urljoin('file:', pathname2url(open_file))
+        else:
+            assembled_url = url_path_join(self.connection_url, uri)
+
+        b = lambda: browser.open(assembled_url, new=self.webbrowser_open_new)
         threading.Thread(target=b).start()
 
     def start_app(self):
@@ -1807,8 +1927,8 @@ class ServerApp(JupyterApp):
         self.write_server_info_file()
         self.write_browser_open_file()
 
-        if self.open_browser:
-            self.launch_browser()
+        # Handle the browser opening.
+        self._handle_browser_opening()
 
         if self.token and self._token_generated:
             # log full URL with generated token, so there's a copy/pasteable link
